import sqlite3

class PostMapper:
    def __init__(self,db_path):
        self.db_path=db_path
        self.connection=sqlite3.connect(self.db_path)
        self.cursor=self.connection.cursor()
        
    def setPath(self,db_path):
        self.db_path=db_path
        self.__init__()
        
    def getAll(self):
        self.cursor.execute("SELECT * FROM posts")
        return self.cursor.fetchall()
    
    def addPost(self,id,title,content,user_id,user_nickname,board_id,board_name,n_views,n_replies,n_comment):
        self.cursor.execute("INSERT INTO posts VALUES(?,?,?,?,?,?,?,?,?,?)",(id,title,content,user_id,user_nickname,board_id,board_name,n_views,n_replies,n_comment))
        self.connection.commit()
        
    def isPostExistById(self,id):
        self.cursor.execute("SELECT * FROM posts WHERE id=?",(id,))
        if self.cursor.fetchall()==[]:
            return False
        else:
            return True
        
    def getAllUserID(self):
        self.cursor.execute("SELECT user_id from posts")
        return self.cursor.fetchall()
        
    def getUsernameById(self,id):
        self.cursor.execute("SELECT user_nickname from posts where user_id=?",(id,))
        res=self.cursor.fetchall()
        if res==[]:
            return None
        else:
            return(res[0][0])
        
    def getAllBoardName(self):
        self.cursor.execute("SELECT board_name from posts")
        return self.cursor.fetchall()

    def getAllTitleAndContent(self):
        self.cursor.execute("SELECT title,content from posts")
        return self.cursor.fetchall()



from .Requester import Requester
from ..DBMapper.NoExistPostsMapper import NoExistPostsMapper
from ..DBMapper.PostMapper import PostMapper
import logging
import time

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
)

class Program:
    def __init__(self,start_id,end_id,sleep_time,db_url):
        self.requester=Requester("https://api.codemao.cn/web/forums/posts/{}/details",'db/data.db')
        self.db_url=db_url
        self.noExistPostMapper=NoExistPostsMapper(self.db_url)
        self.postMapper=PostMapper(self.db_url)
        self.start_id=start_id
        self.end_id=end_id
        self.sleep_time=sleep_time
        self.logger=logging.getLogger(__name__)
        
    def doJob(self):
        for i in range(self.start_id,self.end_id+1):
            if not(self.noExistPostMapper.isExistById(i) or self.postMapper.isPostExistById(i)):
                res=self.requester.requestById(i)
                if res.status=="FAILURE":
                    self.noExistPostMapper.addNoExist(res.id)
                    self.logger.info(f'request id {i},fail')
                elif res.status=="OK":
                    self.postMapper.addPost(res.id,res.title,res.content,res.user_id,res.user_nickname,res.board_id,res.board_name,res.n_views,res.n_replies,res.n_comments)
                    self.logger.info(f'request to {i},PostEntity:{res}')
                time.sleep(self.sleep_time)
            else:
                self.logger.info(f"id {i} is exist in db,skipped")


import sqlite3
import sys

def generate_db(url):
    connection=sqlite3.connect(url)
    cursor=connection.cursor()
    cursor.execute("CREATE TABLE IF NOT EXISTS posts(id INT PRIMARY KEY,title TEXT,content TEXT,user_id TEXT,user_nickname TEXT,board_id INT,board_name TEXT,n_views INT,n_replies INT,n_comments INT);")
    cursor.execute("CREATE TABLE IF NOT EXISTS noExistPosts(id INT PRIMARY KEY);")
    connection.commit()

args=sys.argv[1:]
try:
    generate_db(args[0])
    print(f"成功生成帖子数据库到{args[0]}")
except IndexError:
    print("传参有误：正确顺序为:[数据库URL]")
except:
    print("因为未知原因无法生成")


from codemao_diger.DBMapper.PostMapper import PostMapper
from pyecharts.charts import Bar
from collections import Counter
from codemao_diger.charts.board_bar.BoardEntity import BoardEntity

class BoardBar(Bar):
    def __init__(self,db_url,output_file):
        super().__init__()
        self.db_url=db_url
        self.output_file=output_file
        self.x_axis=[]
        self.y_axis=[]
        self.postMapper=PostMapper(self.db_url)
        
    def genCount(self):
        res=self.postMapper.getAllBoardName()
        boardsData=[]
        for i in res:
            boardsData.append(i[0])
        counter=Counter(boardsData)
        boards=[]
        for i in counter:
            boards.append(BoardEntity(i,counter[i]))
        boards=sorted(boards,key=lambda x:x.post_count,reverse=True)
        for i in boards:
            self.x_axis.append(i.board_name)
            self.y_axis.append(i.post_count)
    def render(self):
        self.genCount()
        self.add_xaxis(self.x_axis)
        self.add_yaxis("帖子数量",self.y_axis)
        super().render(self.output_file)



from pyecharts.charts import WordCloud
from codemao_diger.DBMapper.PostMapper import PostMapper
import jieba
from bs4 import BeautifulSoup
from collections import Counter

class HotWordCloud(WordCloud):
    def __init__(self, db_url,output_file):
        super().__init__()
        self.db_url=db_url
        self.output_file=output_file
        self.postMapper=PostMapper(self.db_url)
        self.word_pair=[]
        
    
    def genWordPair(self):
        words=[]
        res=self.postMapper.getAllTitleAndContent()
        for i in res:
            title_words=jieba.cut(i[0])
            soup=BeautifulSoup(i[1],'html.parser')
            content_words=jieba.cut(soup.get_text())
            for i in title_words:
                if len(i)>1:
                    words.append(i)
            for i in content_words:
                if len(i)>1:
                    words.append(1)
        counter=Counter(words)
        for i in counter:
            self.word_pair.append((i,counter[i]))
    
    def render(self):
        self.genWordPair()
        super().add(series_name='热词统计',data_pair=self.word_pair)
        super().render(self.output_file)


from codemao_diger.charts.board_bar import BoardBar
import sys

args=sys.argv[1:]
try:
    postBar=BoardBar(args[0],args[1])
    postBar.render()

except IndexError:
    print("传参错误，正确顺序为:[数据库URL] [输出文件URL]")


import sys
from codemao_diger.charts.wordcloud import HotWordCloud

try:
    args=sys.argv[1:]
    postBar=HotWordCloud(args[0],args[1])
    postBar.render()
except IndexError:
    print("传参有误，顺序为:[数据库URL] [生成图表URL]")


import sqlite3

class NoExistPostsMapper:
    def __init__(self,db_path):
        self.db_path=db_path
        self.connection=sqlite3.connect(self.db_path)
        self.cursor=self.connection.cursor()
        
    def isExistById(self,id):
        self.cursor.execute("SELECT * FROM noExistPosts WHERE id=?",(id,))
        if self.cursor.fetchall()==[]:
            return False
        else: 
            return True
        
    def addNoExist(self,id):
        self.cursor.execute("INSERT INTO noExistPosts VALUES(?)",(id,))
        self.connection.commit()
        
    def getAll(self):
        self.cursor.execute("SELECT * FROM noExistPosts")
        return self.cursor.fetchall()
        
if __name__=="__main__":
    noExistPostsMapper=NoExistPostsMapper("db/data.db")
    noExistPostsMapper.addNoExist(1)
    print(noExistPostsMapper.isExistById(1))
    print(noExistPostsMapper.getAll())


from codemao_diger.DBMapper.PostMapper import PostMapper

class UserEntity:
    def __init__(self,id,username,n_post,db_url):
        self.id=id
        self.username=username
        self.n_post=n_post
        self.postMapper=PostMapper(db_url)
        
    def reload_username(self):
        self.username=self.postMapper.getUsernameById(self.id)
        
        
if __name__=='__main__':
    user=UserEntity(1,'',10,'/home/seterain3913/文档/codemao-diger-rebuild/db/data.db')
    user.reload_username()
    print(user.username)


from pyecharts.charts import Bar
import sqlite3
from codemao_diger.DBMapper.PostMapper import PostMapper
from collections import Counter
from codemao_diger.charts.post_bar.UserEntity import UserEntity

class PostBar(Bar):
    def __init__(self,db_url,ranktop,path):
        super().__init__()
        self.connection=sqlite3.connect(db_url)
        self.cursor=self.connection.cursor()
        self.postMapper=PostMapper(db_url)
        self.db_url=db_url
        self.x_axis=[]
        self.y_axis=[]    
        self.ranktop=ranktop
        self.path=path
    
    def genCount(self):
        res=self.postMapper.getAllUserID()
        uid_list=[]
        for i in res:
            uid_list.append(i[0])
        counter=Counter(uid_list)
        userList=[]
        for i in counter:
            userList.append(UserEntity(int(i),'',counter[i],self.db_url))
        for i in userList:
            i.reload_username()
        sorted_userList=sorted(userList,key=lambda x:x.n_post,reverse=True)
        for i in sorted_userList:
            self.x_axis.append(i.username)
            self.y_axis.append(i.n_post)
    def render(self):
        self.genCount()
        self.add_xaxis(self.x_axis[:self.ranktop])
        self.add_yaxis("发帖量",self.y_axis[:self.ranktop])
        return super().render(self.path)


from . import Program
import sys

args = sys.argv[1:]
try:
    program=Program(int(args[0]),int(args[1]),float(args[2]),args[3])
    program.doJob()
except IndexError:
    print('传参不正确，顺序为:[起始id] [结束id] [间隔时间] [数据库URL]')


import sys
from codemao_diger.charts.post_bar import PostBar

try:
    args=sys.argv[1:]
    postBar=PostBar(args[0],int(args[1]),args[2])
    postBar.render()
except IndexError:
    print("传参有误，顺序为:[数据库URL] [X轴数量] [生成图表URL]")


class PostEntity:
    def __init__(self,status,id,title,content,user_id,user_nickname,board_id,board_name,n_views,n_replies,n_comments):
        self.status=status
        self.id=id
        self.title=title
        self.content=content
        self.user_id=user_id
        self.user_nickname=user_nickname
        self.board_id=board_id
        self.board_name=board_name
        self.n_views=n_views
        self.n_replies=n_replies
        self.n_comments=n_comments
    
    def __str__(self):
        return f"ResponseEntity(status={self.status},id={self.id},title={self.content},user_id={self.user_id},user_nickname={self.user_nickname},board_id={self.board_id},board_name={self.board_name},n_views={self.n_views},n_replies={self.n_replies},n_comments={self.n_replies})"


#!/usr/bin/env python
#   -*- coding: utf-8 -*-

from setuptools import setup
from setuptools.command.install import install as _install

class install(_install):
    def pre_install_script(self):
        pass

    def post_install_script(self):
        pass

    def run(self):
        self.pre_install_script()

        _install.run(self)

        self.post_install_script()

if __name__ == '__main__':
    setup(
        name = 'codemao-diger-rebuild',
        version = '2.0.3',
        description = '',
        long_description = '# ⚡ 猫站洛阳铲 ⚡\n\n一个自动化在编程猫论坛挖坟的工具，当然也可以作为一个包导入到您的项目中\n\n![GitHub last commit](https://img.shields.io/github/last-commit/Rov-Waff/codemao-diger-rebuild)\n\n![PyPI - Version](https://img.shields.io/pypi/v/codemao-diger-rebuild)\n\n![Dynamic JSON Badge](https://img.shields.io/badge/dynamic/json?url=https%3A%2F%2Fapi.codemao.cn%2Fweb%2Fforums%2Fposts%2F1630334%2Fdetails&query=n_views&label=%E7%8C%AB%E7%AB%99%E7%A4%BE%E5%8C%BA)\n\n![GitHub Repo stars](https://img.shields.io/github/stars/Rov-Waff/codemao-diger-rebuild)\n\n## 安装\n\n### 通过pip安装\n\n``````shell\npip install codemao-diger-rebuild\n``````\n\n### 通过源码安装\n\n```shell\ngit clone git@github.com:Rov-Waff/codemao-diger-rebuild.git\ncd codemao-diger-rebuild\npython3 setup.py install\n```\n\n## 命令行使用\n\n你可以运行这个包提供的一些Model以在猫站挖坟\n\n### 生成帖子数据库\n\n运行`codemao_diger.DBGenerator`这个Model可以生成存储帖子数据用的数据库，用法如下：\n\n```shell\npython3 -m codemao_diger.DBGenerator [数据库URL]\n```\n\n表结构请参考[文档](#)（没写好）\n\n### 自动化挖坟\n\n运行`codemao_diger.requester`这个Model可以自动化请求猫站API获取帖子数据，并将其存在指定数据库中，用法如下：\n\n```shell\npython3 -m codemao_diger.requester [起始ID] [结束ID] [间隔时间] [输出数据库]\n```\n\n### 数据分析（饼）\n\n未完成，开发进度将会提交到`chart_dev`分支下，计划会有以下的功能\n\n#### 生成热词词云图\n\n计划Model路径`codemao_diger.chart.worldcloud`\n\n#### 生成发帖量柱状图\n\n计划Model路径`codemao_diger.chart.post_bar`\n\n#### 生成板块帖子量柱状图\n\n计划Model路径`codemao_diger.chart.board_bar`\n\n## 在您的项目中引用\n\n你也可以在你自己的项目中引用这个包，请遵循MIT License使用，文档暂时未完善\n\n## 贡献\n\n如果你要为项目贡献文档，请直接将写好的文档提交到`main`分支下的`/doc`文件夹\n\n如果你要为数据分析贡献代码，请提交至`chart_dev`分支\n',
        long_description_content_type = None,
        classifiers = [
            'Development Status :: 3 - Alpha',
            'Programming Language :: Python'
        ],
        keywords = '',

        author = '',
        author_email = '',
        maintainer = '',
        maintainer_email = '',

        license = 'MIT License',

        url = 'https://github.com/Rov-Waff/codemao-diger-rebuild',
        project_urls = {},

        scripts = [],
        packages = [
            'codemao_diger.DBMapper',
            'codemao_diger.charts.board_bar',
            'codemao_diger.charts.post_bar',
            'codemao_diger.charts.wordcloud',
            'codemao_diger.requester',
            'codemao_diger.requester.entity'
        ],
        namespace_packages = [],
        py_modules = ['codemao_diger.DBGenerator'],
        entry_points = {},
        data_files = [],
        package_data = {},
        install_requires = [
            'bs4',
            'jieba>=0.40',
            'pyecharts>=2.0.0',
            'requests>=2.32'
        ],
        dependency_links = [],
        zip_safe = True,
        cmdclass = {'install': install},
        python_requires = '',
        obsoletes = [],
    )



import requests,json
from .entity.PostEntity import PostEntity

class Requester:
    def __init__(self,url,db_path):
        self.url=url
        self.db_path=db_path
        
    def isPostExistOnlineById(self,id):
        res=requests.get(self.url.format(id))
        if res.status_code==200:
            return True
        else:
            return False
        
    def requestById(self,id):
        res=requests.get(self.url.format(id))
        if self.isPostExistOnlineById(id):
            res_content=json.loads(res.content)
            post_id=int(res_content['id'])
            title=res_content['title']
            content=res_content['content']
            user_id=int(res_content['user']['id'])
            user_nickname=res_content['user']['nickname']
            board_id=int(res_content['board_id'])
            board_name=res_content['board_name']
            n_views=int(res_content['n_views'])
            n_replies=int(res_content['n_replies'])
            n_comments=int(res_content['n_comments'])
            return PostEntity('OK',post_id,title,content,user_id,user_nickname,board_id,board_name,n_views,n_replies,n_comments)
        
        else:
            return PostEntity('FAILURE',id,'','',0,'',0,'',0,0,0)
        
if __name__=='__main__':
    requester=Requester("https://api.codemao.cn/web/forums/posts/{}/details","db/data.db")
    print(requester.requestById(3))
    print(requester.requestById(72))
    